@@ -19,7 +19,10 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+from pyzil.zilliqa.api import APIError
 
+TXN_HASH_NOT_PRESENT = "Txn Hash not Present"
+TX_BLOCK_NO_TRANSACTIONS = "TxBlock has no transactions"
 
 class ZilliqaService(object):
     def __init__(self, zilliqa_api):
@@ -44,4 +47,24 @@ class ZilliqaService(object):
         return self.zilliqa_api.GetLatestTxBlock()
 
     def get_transactions(self, block_number):
-        return self.zilliqa_api.GetTxnBodiesForTxBlock(str(block_number))
+        try:
+            return self.zilliqa_api.GetTxnBodiesForTxBlock(str(block_number))
+        except APIError as e:
+            if str(e) == TXN_HASH_NOT_PRESENT:
+                return self.get_validated_transactions(block_number)
+            if str(e) == TX_BLOCK_NO_TRANSACTIONS:
+                return []
+            raise e
+
+    def get_validated_transactions(self, block_number):
+        for group in self.zilliqa_api.GetTransactionsForTxBlock(str(block_number)):
+            if isinstance(group, list):
+                for txn_hash in group:
+                    try:
+                        yield self.get_transaction_by_hash(txn_hash)
+                    except APIError as e:
+                        if str(e) != TXN_HASH_NOT_PRESENT:
+                            raise e
+
+    def get_transaction_by_hash(self, txn_hash):
+        return self.zilliqa_api.GetTransaction(txn_hash)
